@@ -13,7 +13,7 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
@@ -48,6 +48,13 @@ enum {
 	KEY_RELEASE_EVENT,
 	LEAVE_NOTIFY_EVENT,
 	MOTION_NOTIFY_EVENT,
+
+	ITEM_ADDED,
+	ITEM_REMOVED,
+
+	VIEW_REGISTER,
+	VIEW_UNREGISTER,
+
 	N_SIGNALS
 };
 
@@ -63,8 +70,6 @@ G_DEFINE_TYPE_WITH_CODE(CcItem, cc_item, G_TYPE_INITIALLY_UNOWNED,
  * @CC_DISPOSED: specifies whether dispose has already been run on an item.
  * @CC_GRID_ALIGNED: specifies whether an item should be rendered to match an
  * existing grid.
- * @CC_NO_SEARCH_CHILDREN: specifies whether calls like cc_item_distance()
- * should be forwarded to children.
  */
 
 /**
@@ -112,15 +117,6 @@ G_DEFINE_TYPE_WITH_CODE(CcItem, cc_item, G_TYPE_INITIALLY_UNOWNED,
  * Returns %TRUE if the flag was set, %FALSE if not.
  */
 
-/**
- * CC_ITEM_NO_SEARCH_CHILDREN:
- * @i: a #CcItem
- *
- * Queries the #CcItem @i whether it has the %CC_NO_SEARCH_CHILDEN flag set.
- *
- * Returns %TRUE if the flag was set, %FALSE if not.
- */
-
 /**
  * cc_item_new:
  *
@@ -146,6 +142,36 @@ cc_item_append(CcItem* self, CcItem* child) {
 	cc_item_insert(self, child, -1);
 }
 
+/**
+ * cc_item_add_view:
+ * @self: a #CcItem
+ * @view: a #CcView
+ *
+ * Registers a view at the item. The item and its children can now set up data
+ * for this view.
+ */
+void
+cc_item_add_view(CcItem* self, CcView* view) {
+	g_return_if_fail(CC_IS_ITEM(self));
+	g_return_if_fail(CC_IS_VIEW(view));
+
+	self->views = g_list_prepend(self->views, view);
+	// FIXME: add a weak reference on the view, and then add a FIXME for
+	// removal in _remove_view()
+	g_signal_emit(self, cc_item_signals[VIEW_REGISTER], 0, view);
+}
+
+void
+cc_item_remove_view(CcItem* self, CcView* view) {
+	g_return_if_fail(CC_IS_ITEM(self));
+	g_return_if_fail(CC_IS_VIEW(view));
+
+	self->views = g_list_remove(self->views, view);
+	// FIXME: add an extra check to make sure that the view is definitely
+	// not in the list anymore (and disable it with G_DISABLE_CHECKS)
+	g_signal_emit(self, cc_item_signals[VIEW_UNREGISTER], 0, view);
+}
+
 static void
 ci_update_all_bounds(CcView const* view, CcItem* self) {
 	CcDRect const* all_bounds      = cc_item_get_all_bounds(self, view);
@@ -282,6 +308,11 @@ cc_item_grab_focus(CcItem* self, CcView* view) {
 	g_signal_emit(self, cc_item_signals[FOCUS_ENTER], 0, view, &ev, &retval);
 }
 
+static void
+ci_add_parent_view_to_child(CcItem* self, CcView* view, gpointer data) {
+	cc_item_view_register(CC_ITEM_VIEW(view), CC_ITEM(data));
+}
+
 /**
  * cc_item_insert:
  * @self: a #CcItem
@@ -300,6 +331,10 @@ cc_item_insert(CcItem* self, CcItem* child, gint position) {
 	self->children = g_list_insert(self->children, g_object_ref_sink(child), position);
 	child->parent = self;
 	cc_item_view_register(CC_ITEM_VIEW(self), child);
+	position = g_list_index(self->children, child);
+	cc_item_foreach_view(self, ci_add_parent_view_to_child, child);
+
+	g_signal_emit(self, cc_item_signals[ITEM_ADDED], 0, position, child);
 }
 
 /**
@@ -328,6 +363,13 @@ cc_item_is_child_of(CcItem const* child, CcItem const* parent) {
 	return retval;
 }
 
+static void
+ci_remove_parent_view_from_child(CcItem* self, CcView* view, gpointer data) {
+	CcItem* child = CC_ITEM(data);
+
+	cc_item_view_unregister(CC_ITEM_VIEW(view), child);
+}
+
 /**
  * cc_item_remove:
  * @self: a #CcItem
@@ -338,10 +380,15 @@ cc_item_is_child_of(CcItem const* child, CcItem const* parent) {
  */
 void
 cc_item_remove(CcItem* self, CcItem* child) {
+	gint position;
 	g_return_if_fail(CC_IS_ITEM(self) && CC_IS_ITEM(child));
 
+	position = g_list_index(self->children, child);
 	self->children = g_list_remove(self->children, child);
+	cc_item_foreach_view(self, ci_remove_parent_view_from_child, child);
 	cc_item_view_unregister(CC_ITEM_VIEW(self), child);
+
+	g_signal_emit(self, cc_item_signals[ITEM_REMOVED], 0, position, child);
 	g_object_unref(child);
 }
 
@@ -449,19 +496,13 @@ ci_dispose(GObject* object) {
 	g_list_foreach(self->children, G_FUNC(ci_remove_swapped), self);
 	// self->children is NULL now
 
+	g_object_unref(self->bounds);
+	g_object_unref(self->all_bounds);
+	g_object_unref(self->children_bounds);
+
 	G_OBJECT_CLASS(cc_item_parent_class)->dispose(object);
 }
 
-static void
-ci_finalize(GObject* object) {
-	CcItem* self = CC_ITEM(object);
-
-	g_free(self->all_bounds);
-	self->all_bounds = NULL;
-
-	G_OBJECT_CLASS(cc_item_parent_class)->finalize(object);
-}
-
 static void
 ci_get_property(GObject* object, guint prop_id, GValue* value, GParamSpec* pspec) {
 	CcItem* self = CC_ITEM(object);
@@ -486,16 +527,22 @@ static gdouble
 ci_distance(CcItem* self, gdouble x, gdouble y, CcItem** found) {
 	gdouble  distance = G_MAXDOUBLE;
 	GList  * child;
-
-	if(CC_ITEM_NO_SEARCH_CHILDREN(self)) {
-		return distance;
-	}
+	CcItem * child_found = NULL;
 
 	g_return_val_if_fail(found && !CC_IS_ITEM(*found), distance);
 
-	for(child = g_list_last(self->children); child && found && !CC_IS_ITEM(*found); child = child->prev) {
-		gdouble new_dist = cc_item_distance(CC_ITEM(child->data), x, y, found);
-		distance = MIN(distance, new_dist);
+	for(child = g_list_last(self->children); child ; child = child->prev) {
+	  child_found = NULL;
+	  gdouble new_dist = cc_item_distance(CC_ITEM(child->data), x, y, &child_found);
+	  //distance = MIN(distance, new_dist);
+
+	  if (distance > new_dist){
+	    distance = new_dist;
+	    if (child_found)
+	      *found = child_found;
+	    else
+	      *found = CC_ITEM(child->data);
+	  }
 	}
 
 	return distance;
@@ -549,6 +596,7 @@ ci_event(CcItem* self, CcView* view, GdkEvent* ev) {
 	}
 
 	if(!retval && self->parent) {
+		// FIXME: make sure item->parent is displayed in view
 		g_signal_emit(self->parent, cc_item_signals[EVENT], 0, view, ev, &retval);
 	}
 
@@ -652,6 +700,29 @@ ci_notify_child_bounds_real(CcItem* self, CcItem* child, CcView* view, CcDRect c
 	}
 }
 
+static void
+ci_child_emit_view_register(CcItem* item, CcView* view) {
+	g_signal_emit(item, cc_item_signals[VIEW_REGISTER], 0, view);
+}
+
+static void
+ci_view_register(CcItem* self, CcView* view) {
+	g_list_foreach(self->children, G_FUNC(ci_child_emit_view_register), view);
+}
+
+static void
+ci_child_emit_view_unregister(CcItem* item, CcView* view) {
+	g_signal_emit(item, cc_item_signals[VIEW_UNREGISTER], 0, view);
+}
+
+static void
+ci_view_unregister(CcItem* self, CcView* view) {
+	if(cc_item_get_all_bounds(self, view)) {
+		cc_item_dirty(self, view, *cc_item_get_all_bounds(self, view));
+	}
+	g_list_foreach(self->children, G_FUNC(ci_child_emit_view_unregister), view);
+}
+
 static void
 cc_item_class_init(CcItemClass* self_class) {
 	GObjectClass* go_class;
@@ -659,7 +730,6 @@ cc_item_class_init(CcItemClass* self_class) {
 	/* GObjectClass */
 	go_class = G_OBJECT_CLASS(self_class);
 	go_class->dispose      = ci_dispose;
-	go_class->finalize     = ci_finalize;
 	go_class->get_property = ci_get_property;
 
 	cc_item_signals[ALL_BOUNDS_CHANGED] =
@@ -839,6 +909,68 @@ cc_item_class_init(CcItemClass* self_class) {
 			     cc_marshal_BOOLEAN__OBJECT_BOXED,
 			     G_TYPE_BOOLEAN, 2,
 			     CC_TYPE_VIEW, GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
+	/**
+	 * CcItem::item-added
+	 * @self: a #CcItem
+	 * @position: the new index of the child
+	 * @child: the child
+	 *
+	 * This signal gets emitted when an element is added to another element.
+	 */
+	cc_item_signals[ITEM_ADDED] =
+		g_signal_new("item-added", CC_TYPE_ITEM,
+			     G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET(CcItemClass, item_added),
+			     NULL, NULL,
+			     cc_marshal_VOID__INT_OBJECT,
+			     G_TYPE_NONE, 2,
+			     G_TYPE_INT, CC_TYPE_ITEM);
+	/**
+	 * CcItem::item-removed
+	 * @self: a #CcItem
+	 * @position: the former index of the child
+	 * @child: the child
+	 *
+	 * This signal gets emitted when an element is removed from another element.
+	 */
+	cc_item_signals[ITEM_REMOVED] =
+		g_signal_new("item-removed", CC_TYPE_ITEM,
+			     G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET(CcItemClass, item_removed),
+			     NULL, NULL,
+			     cc_marshal_VOID__INT_OBJECT,
+			     G_TYPE_NONE, 2,
+			     G_TYPE_INT, CC_TYPE_ITEM);
+	/**
+	 * CcItem::view-register
+	 * @self: a #CcItem
+	 * @view: a #CcView
+	 *
+	 * Registeres the view for this item. This signal can be used to create
+	 * view-specific data for the #CcItem. Take a look at #CcText to see
+	 * how this can be used.
+	 */
+	cc_item_signals[VIEW_REGISTER] =
+		g_signal_new("view-register", CC_TYPE_ITEM,
+			     G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET(CcItemClass, view_register),
+			     NULL, NULL,
+			     g_cclosure_marshal_VOID__OBJECT,
+			     G_TYPE_NONE, 1,
+			     CC_TYPE_VIEW);
+	/**
+	 * CcItem::view-unregister
+	 * @self: a #CcItem
+	 * @view: a #CcView
+	 *
+	 * Registeres the view for this item. This signal can be used to create
+	 * view-specific data for the #CcItem. Take a look at #CcText to see
+	 * how this can be used.
+	 */
+	cc_item_signals[VIEW_UNREGISTER] =
+		g_signal_new("view-unregister", CC_TYPE_ITEM,
+			     G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET(CcItemClass, view_unregister),
+			     NULL, NULL,
+			     g_cclosure_marshal_VOID__OBJECT,
+			     G_TYPE_NONE, 1,
+			     CC_TYPE_VIEW);
 
 	/* CriaItemClass */
 	self_class->distance            = ci_distance;
@@ -846,6 +978,9 @@ cc_item_class_init(CcItemClass* self_class) {
 	self_class->focus               = ci_focus;
 	self_class->render              = ci_render;
 	self_class->notify_child_bounds = ci_notify_child_bounds_real;
+
+	self_class->view_register       = ci_view_register;
+	self_class->view_unregister     = ci_view_unregister;
 }
 
 /* CcItemViewIface */
