@@ -28,6 +28,7 @@
 #include "demo-window.h"
 #include "fifteen.h"
 #include "line.h"
+#include "main.h"
 #include "planets.h"
 #include "text-demo.h"
 #include "tree-integration.h"
@@ -46,6 +47,7 @@ main(int argc, char* argv[]) {
 	demo_window_add(DEMO_WINDOW(mainwin), crossing());
 	demo_window_add(DEMO_WINDOW(mainwin), fifteen());
 	demo_window_add(DEMO_WINDOW(mainwin), line());
+	demo_window_add(DEMO_WINDOW(mainwin), pixbuf_demo());
 	demo_window_add(DEMO_WINDOW(mainwin), planet_page());
 	demo_window_add(DEMO_WINDOW(mainwin), text_demo());
 	//demo_window_add(DEMO_WINDOW(mainwin), tree_integration());
