@@ -98,8 +98,8 @@ update_entry_from_layout(CcText* text, GParamSpec* pspec, GtkEntry* entry) {
 	g_object_freeze_notify(G_OBJECT(entry));
 }
 
-CcItem* root = NULL;
-CcItem* arrow = NULL;
+static CcItem* root = NULL;
+static CcItem* arrow = NULL;
 
 DemoPage*
 text_demo(void) {
